package me.earth.havoxrestart;

import me.earth.earthhack.api.command.Command;
import me.earth.earthhack.api.event.bus.instance.Bus;
import me.earth.earthhack.api.plugin.Plugin;
import me.earth.earthhack.api.register.exception.AlreadyRegisteredException;
import me.earth.earthhack.impl.event.events.misc.TickEvent;
import me.earth.earthhack.impl.event.listeners.LambdaListener;
import me.earth.earthhack.impl.managers.Managers;
import me.earth.earthhack.impl.util.math.StopWatch;
import me.earth.earthhack.impl.util.text.ChatIDs;
import me.earth.earthhack.impl.util.text.TextColor;
import me.earth.earthhack.pingbypass.PingBypass;
import net.minecraft.client.Minecraft;

import java.util.concurrent.TimeUnit;

@SuppressWarnings("unused")
public class HavoxRestart implements Plugin {
    private final long ms;
    private final StopWatch restartTimer = new StopWatch();
    private final StopWatch onConnectTimer = new StopWatch();
    private final StopWatch announceTimer = new StopWatch();
    private long announceInterval = TimeUnit.HOURS.toMillis(1);
    private boolean announceOnConnect = false;
    private boolean wasConnected = false;

    public HavoxRestart() {
        if (!PingBypass.CONFIG.isLoaded()) {
            PingBypass.CONFIG.load();
        }

        this.ms = Long.parseLong(PingBypass.CONFIG.getProperty("havox.delay", "21600")) * 1000L;
    }

    @Override
    public void load() {
        restartTimer.reset();
        onConnectTimer.reset();
        announceTimer.reset();

        if (PingBypass.isServer()) {
            try {
                Managers.COMMANDS.register(new Command(new String[][]{{"restart-time"}}) {
                    @Override
                    public void execute(String[] args) {
                        sendLeftTime();
                    }
                });
            } catch (AlreadyRegisteredException e) {
                e.printStackTrace();
            }

            String message = getMessage();
            System.out.println(message);

            Bus.EVENT_BUS.register(new LambdaListener<>(TickEvent.class, e -> {
                if (restartTimer.passed(ms)) {
                    Minecraft.getMinecraft().shutdown();
                    return;
                }

                boolean isConnected = PingBypass.isConnected();
                if (!wasConnected && (wasConnected = isConnected)) {
                    announceOnConnect = true;
                    onConnectTimer.reset();
                } else if (!isConnected) {
                    wasConnected = false;
                    announceOnConnect = false;
                }

                if (ms - restartTimer.getTime() < TimeUnit.SECONDS.toMillis(11)) {
                    announceInterval = TimeUnit.SECONDS.toMillis(1);
                } else if (ms - restartTimer.getTime() < TimeUnit.MINUTES.toMillis(10)) {
                    announceInterval = TimeUnit.MINUTES.toMillis(1);
                } else if (ms - restartTimer.getTime() < TimeUnit.MINUTES.toMillis(60)) {
                    announceInterval = TimeUnit.MINUTES.toMillis(10);
                }

                if (announceOnConnect && onConnectTimer.passed(5_000)) {
                    announceOnConnect = false;
                    sendLeftTime();
                }

                if (announceTimer.passed(announceInterval - 250)) {
                    sendLeftTime();
                    announceTimer.reset();
                }
            }));
        }
    }

    private void sendLeftTime() {
        String message = getMessage();
        System.out.println(message);
        Managers.CHAT.sendDeleteMessage(message, "restart-time", ChatIDs.COMMAND);
    }

    private String getMessage() {
        long millis = ms - restartTimer.getTime();
        return String.format(TextColor.YELLOW + "PingBypass will restart in %02d:%02d:%02d",
                             TimeUnit.MILLISECONDS.toHours(millis),
                             TimeUnit.MILLISECONDS.toMinutes(millis) - TimeUnit.HOURS.toMinutes(TimeUnit.MILLISECONDS.toHours(millis)),
                             TimeUnit.MILLISECONDS.toSeconds(millis) - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(millis)));
    }

}
